#!/usr/bin/env python
# encoding: iso-8859-15
#
#Min. Mundo Interactivo-Narrativo en Espaol [Tierra-Media]
#Copyright (C) 2002  Pablo Ruiz Mzquiz
#
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#Fichero: Opcion.py

"""Clase Opcion."""

import logging
from random import random
import os

from utils import *
import Guion
import FechaHoraMine
import CronoMine
import xmlutils

log = logging.getLogger("Modulos")

class Opcion(object):
    """Esta clase representa una opcin de mdulo, es decir, un guin con algunos atributos
    aadidos para definir cundo y cmo se debe activar este guin y no otro:
    
    - Recuerda cuantas veces ha sido utilizado recientemente.
    - Tambien se ocupa de "olvidar" el uso del guion con el paso del tiempo.
    - Lee y almacena los "contextos" a los que pertenece el guion.

    La misin de esta clase es doble:

    a) "Parsear" una opcin a partir de un nodo XML contenido en un mdulo.
    b) Ejecutar el guin, recordando las veces que se ha ejecutado.
    """

    # Atributos pblicos de la clase

    def __leer_contexto(self): return self.__contexto 

    contexto = property(__leer_contexto,
                  doc="""(string): permite asociar diferentes comportamientos, sirve para
		      saber si los interlocutores o usuarios de la entidad propietaria de este
		      mdulo siguen una secuencia lgica de acciones o no.
                      """)

    def __leer_duracion_recuerdo(self): return self.__duracion_recuerdo 

    duracion_recuerdo = property(__leer_duracion_recuerdo,
                  doc="""(string): es el nmero de horas Min que transcurren hasta que los
		      usos del guin se "olvidan" y se puede activar el guin de nuevo.
                      """)

    def __leer_repeticiones(self): return self.__repeticiones 

    repeticiones = property(__leer_repeticiones,
                  doc="""("<nmero>" | "siempre" | "nunca"): Un valor numrico indica la cantidad de
		      veces seguidas que se puede repetir el guin. Cuando se gastan todas, no
		      se puede activar de nuevo hasta que los usos se olvidan.

		      Si vale "siempre", siempre se puede activar el guin (no se gasta).

		      Si vale "nunca", el guin es de UN SLO USO, slo se puede activar una
		      vez y sta activacin nunca se olvida.
                      """)
    
    def __leer_guion(self): return self.__guion 

    guion = property(__leer_guion,
                     doc="""(Guion): El guin correspondiente.
		         
		         Invariantes:
			   guion != None
			 """)


    # Constructor

    def __init__(self, contexto, duracion_recuerdo, repeticiones, guion):
        """Crea una nueva opcin a partir de sus atributos.

        Requiere:
            Los parmetros satisfacen las invariantes de los atributos.
        """

	# Atributos privados:
	#
	# __repeticiones_iniciales: guarda el valor inicial de repeticiones, para restaurarlo
	#                           cuando los usos se olvidan.
	# __temporizador: un crono para marcar el momento en que se olvidarn los usos.
	# __cb_finalizar (funcion): funcin llamada al terminar el guin.

	self.__contexto = contexto
	self.__duracion_recuerdo = duracion_recuerdo
	self.__repeticiones = repeticiones
	self.__guion = guion
	self.__repeticiones_iniciales = repeticiones
	self.__temporizador = None
	self.__cb_finalizar = None


    # Mtodos normales
    
    def agotada(self):
	"""Indica si se han consumido el mximo de repeticiones."""
	if self.__repeticiones == "siempre" or self.__repeticiones == "nunca":
	    return 0
     
	return (int(self.__repeticiones) == 0)

	
    def ejecutar(self, macros_valores, cb_finalizar = None):
	"""Activa el guion si le quedan usos disponibles, y lleva la cuenta del
	nmero de repeticiones.
	"""
	if self.__repeticiones != "siempre":
	    if self.__repeticiones == "nunca":
		self.__repeticiones = "0"
		# Al no activar el temporizador, nunca volvera a haber repeticiones disponibles.
	    elif self.__repeticiones > 0:
		self.__repeticiones = str(int(self.__repeticiones) - 1)
		self.__activar_temporizador()
	    else:
		return 0    # Repeticiones agotadas
	
	log.debug("Una repeticion gastada, quedan" + repr(self.__repeticiones))

	if cb_finalizar == None:
	    # Ejecucin instantnea
	    return self.__guion.ejecutar(macros_valores)
	else:
	    self.__cb_finalizar = cb_finalizar
	    self.__guion.ejecutar(macros_valores, self.__fin_ejecucion)


    def cancelar(self):
	"""Cancela la ejecucin. No se llama a la funcin de callback."""
	self.__guion.cancelar()
	self.__cb_finalizar = None


    # Mtodos privados

    def __fin_ejecucion(self):
	"""Registra que ha terminado la ejecucin y llama al callback."""
	if self.__cb_finalizar != None:
	    self.__cb_finalizar()
	self.__cb_finalizar = None

    
    def __activar_temporizador(self):
	"""Establece el tiempo necesario para "olvidar" los usos del guion"""
	import Mundo
	
	if self.__temporizador:
	    self.__temporizador.parar() 

	horas_olvido = FechaHoraMine.crear_rel( segundos = (random() + 0.5) * \
		                                           self.__duracion_recuerdo * \
		                                           FechaHoraMine.SEGUNDOS_POR_HORA )

	log.debug("Activado temporizador para dentro de %s horas Mine" % repr(horas_olvido))
	self.__temporizador = CronoMine.crear(Mundo.cronoMine_control, horas_olvido, 
					      self.__olvidar_repeticiones)

	
    def __olvidar_repeticiones(self, crono):
	if self.__guion.id == None:
	    log.debug("Olvidado guion inline")
	else:
	    log.debug("Olvidado guion %s" % self.__guion.id)

	self.__repeticiones = self.__repeticiones_iniciales
	self.__temporizador = None
    

# Funciones pblicas

def nueva_xml(elem_opcion, macros_tipos, macros_extra):
    """Crea una opcin a partir de un nodo XML de opcin.

    La estructura del nodo es:

      ...
      <opcion contexto="xxx" duracion_recuerdo="xxx" repeticiones="xxx">
        ...guion...
      </opcion>
      ...

    Las macros sern pasadas tal cual al guin interno.

    Requiere:
      elem_opcion != None
      macros_tipos != None
      macros_extra != None
    """
    log.debug("Creando opcin")
    contexto = xmlutils.atributo_def(elem_opcion, 'contexto', 'inicial')
    duracion_recuerdo = xmlutils.atributo_def(elem_opcion, 'duracion_recuerdo', 24)
    repeticiones = xmlutils.atributo_def(elem_opcion, 'repeticiones', 1)
    guion = Guion.nuevo_xml(elem_opcion, macros_tipos, macros_extra)

    opcion = Opcion(contexto, duracion_recuerdo, repeticiones, guion)
    log.debug("Opcion creada correctamente")

    return opcion


# Cdigo para pruebas unitarias del mdulo.

if (__name__ == '__main__'):
    print "No hay prueba unitaria de mdulo. Prueba con objeto.py" 

